<?php

function excart_settings_page($form, $form_state) {
  $form['excart_node_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Node type selection'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['excart_node_settings']['excart_node_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Select node types'),
    '#default_value' => variable_get('excart_node_types', array()),
    '#options' => node_type_get_names(),
    '#ajax' => array(
      'wrapper' => 'excart-type-wrapper',
      'callback' => 'excart_ajax_select'
    )
  );
  $types = variable_get('excart_node_types', array());
  if (isset($form_state['values']['excart_node_types'])) {
    $types = array_filter($form_state['values']['excart_node_types']);
  }
  $form['excart_node_settings']['excart_price_field'] = array(
    '#type' => 'select',
    '#title' => t('Select price field'),
    '#default_value' => variable_get('excart_price_field', 0),
    '#options' => empty($types) ? array() : _excart_get_price_fields($types),
    '#prefix' => '<div id="excart-type-wrapper">',
    '#suffix' => '</div>',
    '#validated' => TRUE
  );
  $form['excart_count_calc'] = array(
    '#title' => t('Select qty calculation algorithm'),
    '#type' => 'select',
    '#options' => array(
      0 => t('By position'),
      1 => t('By pcs'),
    ),
    '#default_value' => variable_get('excart_count_calc', 0),
  );
  $form['#submit'][] = 'excart_settings_page_submit';
  return system_settings_form($form);
}

function excart_settings_page_submit(&$form, &$form_state) {
  $types = $form_state['values']['excart_node_types'];
  $field_name = 'excart_add_to_cart';
  foreach ($types as $type => $active) {
    $instance = field_info_instance('node', $field_name, $type);
    if ($active && !$instance) {
      $instance = array(
        'field_name' => $field_name,
        'label' => 'Add to cart',
        'description' => 'Add to cart link',
        'entity_type' => 'node',
        'bundle' => $type,
      );
      // It doesn't exist. Create it.
      field_create_instance($instance);
    }
    elseif (!$active && $instance) {
      field_delete_instance($instance);
    }
  }
}

function excart_ajax_select(&$form, &$form_state) {
  return $form['excart_node_settings']['excart_price_field'];
}

function _excart_get_price_fields($types = array()) {
  $available_fields = array();
  if (!$types) {
    $types = variable_get('excart_node_types', array());
  }
  if ($types) {
    foreach ($types as $type) {
      $fields = field_info_instances('node', $type);
      foreach ($fields as $name => $field) {
        if (_excart_check_bundles($types, $name)) {
          $available_fields[$name] = $field['label'];
        }
      }
    }
  }
  return $available_fields;
}

function _excart_check_bundles($types, $field_name) {
  $exists = TRUE;
  $field = field_info_field($field_name);
  foreach ($types as $type) {
    if (!in_array($type, $field['bundles']['node'])) {
      $exists = FALSE;
      break;
    }
  }
  return $exists;
}
